Package: MIRA
Version: 1.32.0
Date: 2021-11-21
Title: Methylation-Based Inference of Regulatory Activity
Description: DNA methylation contains information about the regulatory state of the cell.
	MIRA aggregates genome-scale DNA methylation data into a DNA methylation profile
	for a given region set with shared biological annotation. Using this profile,
	MIRA infers and scores the collective regulatory activity for the region set.
	MIRA facilitates regulatory analysis in situations where classical regulatory
	assays would be difficult and allows public sources of region sets to be
	leveraged for novel insight into the regulatory state of DNA methylation
	datasets.
Authors@R: c(person("Nathan", "Sheffield", role=c("aut")),
	person("John", "Lawson", email="jtl2hk@virginia.edu", role=c("aut", "cre")),
	person("Christoph", "Bock", role=c("ctb")))
Author: Nathan Sheffield <http://www.databio.org> [aut],
    Christoph Bock [ctb],
    John Lawson [aut, cre]
Maintainer: John Lawson <jtl2hk@virginia.edu>
Depends: R (>= 3.5)
Imports: BiocGenerics, S4Vectors, IRanges, GenomicRanges, data.table,
        ggplot2, Biobase, stats, bsseq, methods
Suggests: knitr, parallel, testthat, BiocStyle, rmarkdown,
        AnnotationHub, LOLA
VignetteBuilder: knitr
License: GPL-3
biocViews: ImmunoOncology, DNAMethylation, GeneRegulation,
        GenomeAnnotation, SystemsBiology, FunctionalGenomics, ChIPSeq,
        MethylSeq, Sequencing, Epigenetics, Coverage
URL: http://databio.org/mira
BugReports: https://github.com/databio/MIRA
RoxygenNote: 6.1.0
git_url: https://git.bioconductor.org/packages/MIRA
git_branch: RELEASE_3_22
git_last_commit: bb55c9a
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:06:51 UTC; biocbuild
